
type t =
  | NormalQuadratic of Complex.t * Complex.t
  | MonicQuadratic of Complex.t * Complex.t
  | NormalCubic of Complex.t * Complex.t
  | MonicCubic of Complex.t * Complex.t * Complex.t

(* 複素数の計算 *)
let ( +@ ) z1 z2 = Complex.add z1 z2
let ( -@ ) z1 z2 = Complex.sub z1 z2
let ( *@ ) z1 z2 = Complex.mul z1 z2
let ( /@ ) z1 z2 = Complex.div z1 z2

let i y = { Complex.re = 0.; Complex.im = y }

let ( +< ) x z =
  let x_c = { Complex.re = x; Complex.im = 0. } in
  x_c +@ z

let ( *< ) x z =
  let x_c = { Complex.re = x; Complex.im = 0. } in
  x_c *@ z

let omega = (1. /. 2.) *< (-1. +< i (sqrt 3.))

let re { Complex.re; Complex.im } = re

(* x の 3 乗根の1つを計算 *)
let curt x =
  let x_norm = Complex.norm x in
  (* *)
  if x_norm = 0. then Complex.zero
  else if re x = x_norm then (x_norm ** (1. /. 3.)) +< (i 0.)
  else if re x = -. x_norm then (-. (x_norm ** (1. /. 3.))) +< i 0.
  else Complex.polar ((Complex.norm x) ** (1./.3.)) ((Complex.arg x) /. 3.)

let rec solve eqt =
  match eqt with
  | NormalQuadratic (p,q)
    ->  [| p +@ Complex.sqrt q; p -$ Complex.sqrt q |]
  | MonicQuadratic (p,q)
    -> begin 
      let p_nq = (-. 1. /. 2.) *< p in
      let q_nq = (1. /. 4.) *< (p *@ p -$ 4. *< q) in
      solve (NormalQuadratic (p_nq,q_nq))
    end
  | NormalCubic (p,q)
    -> begin
      let p_mq = q in
      let q_mq = (-1. /. 27.) *< (p *@ p *@ p) in
      let r_cube = solve (MonicQuadratic (p_mq,q_mq)) in
      let r = Array.map curt r_cube in
      [| r.(0) +@ r.(1);
	 (omega *@ r.(0)) +@ (omega *@ omega *@ r.(1));
	 (omega *@ omega *@ r.(0)) +@ (omega *@ r.(1)) |]
    end
  | MonicCubic (a,b,c)
    -> begin
      let p = -1. *< (a *@ a) +@ c in
      let q = (2. /. 27.) *< (a *@ a *@ a) -@ (1. /. 3.) *< (a *@ b) +@ c in
      let r = solve (NormalCubic (p,q)) in
      Array.map (fun z -> z -@ (1. /. 3.) *< a) r
    end
      
