(* 複素数の計算 *)
(* 実数 + 複素数 *)
let ( +< ) x z = 
  let x_c = { Complex.re = x; Complex.im = 0. } in
  Complex.add x_c z

let ( -< ) x z = 
  let x_c = { Complex.re = x; Complex.im = 0. } in
  Complex.sub x_c z

let ( *< ) x z =
  let x_c = { Complex.re = x; Complex.im = 0. } in
  Complex.mul x_c z

(* 純虚数 *)
let i y = { Complex.re = 0.; Complex.im = y } 

(* 複素数の演算  *)
let ( +@ ) z1 z2 = Complex.add z1 z2
let ( -@ ) z1 z2 = Complex.sub z1 z2
let ( *@ ) z1 z2 = Complex.mul z1 z2
let ( /@ ) z1 z2 = Complex.div z1 z2


(* 連立方程式 : Ax = b を解く *)
let solve (a11, a12, a21, a22) (b1, b2) =
  let det = (a11 *@ a22) -@ (a12 *@ a21) in
  let (a11', a12', a21', a22') = (a22 /@ det, -1. *< (a21 /@ det), -1. *< (a12 /@ det), a11 /@ det) in
  ((a11' *@ b1) +@ (a12' *@ b2), (a21' *@ b1) +@ (a22' *@ b2))

(*
  2 面バランス取りの計算
  参考資料 : 機械振動工学
 *)
    
(* 影響係数行列 *)
(* a11 u1 = f1_0, a12 u2 = f2_0 *)
(* a11 (u1 + du1) = f1_1, a12 (u2 + du1) = f2_1 *)
(* a21 (u2 + du2) = f1_2, a22 (u2 + du2) = f2_2 *)
let matrix_alpha init test1 test2 =
  let (du_0, f1_0, f2_0) = init in
  let (du_1, f1_1, f2_1) = test1 in
  let (du_2, f1_2, f2_2) = test2 in

  (* a11 du1 = df11, a12 du1 = df12 *)
  (* a21 du2 = df21, a22 du2 = df22 *)
  let dm1 = du_1 in
  let df11 = Complex.sub f1_1 f1_0 in
  let df12 = Complex.sub f2_1 f2_0 in
  let dm2 = du_2 in
  let df21 = Complex.sub f1_2 f2_0 in
  let df22 = Complex.sub f2_2 f2_0 in

  (* 各成分 a11, a12, a21, a22 を計算して影響係数行列を計算 *)
  let a11 = Complex.div df11 dm1 in
  let a12 = Complex.div df12 dm1 in
  let a21 = Complex.div df21 dm2 in
  let a22 = Complex.div df22 dm2 in

  (a11, a12, a21, a22)
 

(* 残留バランス *)
let unbalance init test1 test2 =
  let a = matrix_alpha init test1 test2 in
  let (dm_0, f1_0, f2_0) = init in
  let b = (f1_0, f2_0) in
  solve a b

