
(*

 C exp_2pi_i f t ~ (C, f)
 (C0, f) + (C1, f) = (C0 + C1, f)
 d (C, f) = (i_2pi * C, f)
 d ... d (C, f) = (i_2pi * ... * i_2pi * C, f)
 *)

Fixpoint diff {F : Type} (n : nat) (mul_f : F -> F -> F) (u : F*F) : F*F :=
  let (c,w) := u in
  match n with
    | O => u
    | S n' => diff n' mul_f (mul_f w c, w)
  end.

Lemma order_1st {F : Type} : forall (c w : F) (mul_f : F -> F -> F), diff 1 mul_f (c,w) = ((mul_f w c), w).
Proof.
  intros; simpl; reflexivity.
Qed.

Lemma order_2nd {F : Type} : forall (c w : F) (mul_f : F -> F -> F), diff 2 mul_f (c, w) = ((mul_f w (mul_f w c)), w).
Proof.
  intros; simpl; reflexivity.
Qed.

Definition coefficient {F : Type} (u : F*F) : F := fst u.

Lemma coefficient_base {F : Type} : forall (c w : F), coefficient (c,w) = c.
Proof.
  intros.
  reflexivity.
Qed.

Lemma coefficient_scalar {F : Type} : forall (a c w : F) (mul_f : F -> F -> F), coefficient (mul_f a c, w) = mul_f a c.
Proof.
  intros.
  simpl.
  reflexivity.
Qed.

Fixpoint pow {F : Type} (mul_f : F -> F -> F) (n : nat) (a r : F) : F :=
  match n with
    | O => a
    | S n' => mul_f r (pow mul_f n' a r)
  end.


Lemma diff_coeff_sn {F : Type} : forall (n : nat) (mul_f : F -> F -> F) (c w : F), coefficient (diff n mul_f (mul_f w c, w)) = mul_f w (coefficient (diff n mul_f (c, w))) -> coefficient (diff (S n) mul_f (c, w)) = mul_f w (coefficient (diff n mul_f (c, w))).
Proof.
  intros.
  simpl.
  rewrite H.
  now reflexivity.
Qed.

